import 'dart:core';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

void main() {
  runApp(GetMaterialApp(home: First()));
}

class First extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        leading: IconButton(
          icon: Icon(Icons.add),
          onPressed: () {
            Get.snackbar("Hi", "I'm modern snackbar");
          },
        ),
        title: Text('First Route'),
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            GetBuilder<Controller>(
                // You only need to initialize your controller the first time you use it. Don't use init in your other GetBuilders anymore
                init: Controller(),
                builder: (s) => Text(
                      'clicks: ${s.count}',
                    )),
            RaisedButton(
              child: Text('Next Route'),
              onPressed: () {
                // use Get.to to navigate to Second Screen
                Get.to(Second());
              },
            ),
          ],
        ),
      ),
      floatingActionButton: FloatingActionButton(
          child: Icon(Icons.add),
          onPressed: () {
            Controller.to.increment();
          }),
    );
  }
}

class Second extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('second Route'),
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            GetX<ReactiveController>(
              init: ReactiveController(),
              builder: (_) {
                print("count rebuild");
                return Text('${_.count.value}');
              },
            ),
            GetX<ReactiveController>(
              builder: (_) {
                print("count1 rebuild");
                return Text('${_.count1.value}');
              },
            ),
            GetX<ReactiveController>(
              builder: (_) {
                print("count2 rebuild");
                return Text('${_.count2.value}');
              },
            ),
            GetX<ReactiveController>(
              builder: (_) {
                print("soma rebuild");
                return Text('${_.soma}');
              },
            ),
            GetX<ReactiveController>(
              builder: (_) => Text(_.name.value),
            ),
            RaisedButton(
              child: Text("Go to last page"),
              onPressed: () {
                Get.to(Third());
              },
            ),
            RaisedButton(
              child: Text("Increment"),
              onPressed: () {
                Get.find<ReactiveController>().increment();
              },
            ),
            RaisedButton(
              child: Text("Increment"),
              onPressed: () {
                Get.find<ReactiveController>().count1.value++;
              },
            ),
            RaisedButton(
              child: Text("Increment"),
              onPressed: () {
                Get.find<ReactiveController>().count2.value = 1;
              },
            ),
          ],
        ),
      ),
      floatingActionButton: FloatingActionButton(
          child: Icon(Icons.add),
          onPressed: () {
            Controller.to.increment();
          }),
    );
  }
}

class Third extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      floatingActionButton: FloatingActionButton(onPressed: () {
        Get.find<ReactiveController>().list.add('item');
      }),
      appBar: AppBar(
        title: Text("Third Route"),
      ),
      body: Center(
          // you can use this syntax too
          child: ListViewX<ReactiveController>(
              list: (_) => _.list,
              builder: (context, index) {
                var list = context.list;
                return Text(list[index]);
              })),
    );
  }
}

class Controller extends GetController {
  /// You definitely don't need to use this method.
  /// I use it because it facilitates a lot in productivity
  /// when I have dozens of references to a controller.
  /// In order to use the Get.find<Controller>().count syntax
  /// you can take advantage of the IDE's autocomplete
  /// and type without typing the type like this:
  /// Controller.to.count
  ///

  static Controller get to => Get.find();

  int count = 0;
  void increment() {
    count++;

    /// use update method to update all count variables
    update(this);
  }

  @override
  void close() {
    // close streams here if you need
    super.close();
  }
}

class ReactiveController extends RxController {
  final count = 0.obs;

  final count1 = 0.obs;

  final count2 = 0.obs;

  final list = [0, 1, 2].obs;

  int get soma => count1.value + count2.value;

  final name = "Jonatas Borges".obs;

  increment() {
    count.value++;
    list.addIf(count.value < 10, count.value);
  }
}
